"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getMockContext = void 0;
/**
 * A mock that uses proxies to create properties as they are accessed. Supports
 * function calls and toPrimitive.
 *
 * @param specifics Allows specifying specific values for properties. If a prop
 *  exists on `specifics`, then that value will be used when the property is
 * accessed on automock.
 * @returns
 */
var getAutomock = function (specifics) {
    var autmockSeed = jest.fn();
    var handler = {
        get: function (target, prop) {
            if (prop === Symbol.toPrimitive) {
                return function () { return "Automock: ".concat(JSON.stringify(specifics)); };
            }
            if (specifics && Reflect.has(specifics, prop)) {
                var value = Reflect.get(specifics, prop);
                return value;
            }
            if (!Reflect.has(target, prop)) {
                Reflect.set(target, prop, getAutomock());
            }
            return Reflect.get(target, prop);
        },
        apply: function () {
            return getAutomock();
        },
    };
    return new Proxy(autmockSeed, handler);
};
var getMockContext = function () {
    var mockContext = getAutomock({
        getParameter: jest.fn(function (name) {
            if (name === mockContext.VERSION)
                return "Fake version!";
            return getAutomock();
        }),
    });
    return mockContext;
};
exports.getMockContext = getMockContext;
//# sourceMappingURL=testUtils.js.map